@@ -11,8 +11,8 @@
  *     Mikael Lagerkvist, 2009
  *
  *  Last modified:
- *     $Date: 2013-01-29 17:43:05 +0100 (Tue, 29 Jan 2013) $ by $Author: schulte $
- *     $Revision: 13241 $
+ *     $Date: 2013-07-05 02:04:42 +0200 (Fri, 05 Jul 2013) $ by $Author: tack $
+ *     $Revision: 13808 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -1146,8 +1146,9 @@ namespace Gecode { namespace FlatZinc {
 
     void p_precede(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
       IntVarArgs x = s.arg2intvarargs(ce[0]);
-      IntArgs c = s.arg2intargs(ce[1]);
-      precede(s,x,c,s.ann2icl(ann));
+      int p_s = ce[1]->getInt();
+      int p_t = ce[2]->getInt();
+      precede(s,x,p_s,p_t,s.ann2icl(ann));
     }
 
     void p_nvalue(FlatZincSpace& s, const ConExpr& ce, AST::Node* ann) {
@@ -1624,6 +1625,13 @@ namespace Gecode { namespace FlatZinc {
       SetVarArgs y = s.arg2setvarargs(ce[1],yoff);
       channel(s, x, y);
     }
+
+    void p_precede_set(FlatZincSpace& s, const ConExpr& ce, AST::Node*) {
+      SetVarArgs x = s.arg2setvarargs(ce[0]);
+      int p_s = ce[1]->getInt();
+      int p_t = ce[2]->getInt();
+      precede(s,x,p_s,p_t);
+    }
     
     class SetPoster {
     public:
@@ -1675,6 +1683,7 @@ namespace Gecode { namespace FlatZinc {
         registry().add("gecode_set_weights",
                        &p_weights);
         registry().add("gecode_inverse_set", &p_inverse_set);
+        registry().add("gecode_precede_set", &p_precede_set);
       }
     };
     SetPoster __set_poster;
@@ -1682,6 +1691,12 @@ namespace Gecode { namespace FlatZinc {
 
 #ifdef GECODE_HAS_FLOAT_VARS
 
+    void p_int2float(FlatZincSpace& s, const ConExpr& ce, AST::Node*) {
+      IntVar x0 = s.arg2IntVar(ce[0]);
+      FloatVar x1 = s.arg2FloatVar(ce[1]);
+      channel(s, x0, x1);
+    }
+
     void p_float_lin_cmp(FlatZincSpace& s, FloatRelType frt,
                          const ConExpr& ce, AST::Node*) {
       FloatValArgs fa = s.arg2floatargs(ce[0]);
@@ -1840,6 +1855,7 @@ namespace Gecode { namespace FlatZinc {
     class FloatPoster {
     public:
       FloatPoster(void) {
+        registry().add("int2float",&p_int2float);
         registry().add("float_abs",&p_float_abs);
         registry().add("float_sqrt",&p_float_sqrt);
         registry().add("float_eq",&p_float_eq);
