@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2013-03-05 15:37:20 +0100 (Tue, 05 Mar 2013) $ by $Author: schulte $
- *     $Revision: 13437 $
+ *     $Date: 2013-07-11 12:30:18 +0200 (Thu, 11 Jul 2013) $ by $Author: schulte $
+ *     $Revision: 13840 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -54,11 +54,6 @@ namespace Gecode {
    * Actor
    *
    */
-  size_t
-  Actor::allocated(void) const {
-    return 0;
-  }
-
   Actor* Actor::sentinel;
 
 #ifdef __GNUC__
@@ -79,6 +74,29 @@ namespace Gecode {
   }
 
 
+  /*
+   * No-goods
+   *
+   */
+  void
+  NoGoods::post(Space&) {
+  }
+
+
+  /*
+   * Brancher
+   *
+   */
+  NGL*
+  Brancher::ngl(Space&, const Choice&, unsigned int) const {
+    return NULL;
+  }
+
+  void 
+  Brancher::print(const Space&, const Choice&, unsigned int,
+                  std::ostream&) const {
+  }
+
 
   /*
    * Space: Misc
@@ -114,19 +132,62 @@ namespace Gecode {
   }
 
   void
-  Space::d_resize(void) {
-    if (d_fst == NULL) {
-      // Create new array
-      d_fst = alloc<Actor*>(4);
-      d_cur = d_fst;
-      d_lst = d_fst+4;
-    } else {
-      // Resize existing array
-      unsigned int n = static_cast<unsigned int>(d_lst - d_fst);
-      assert(n != 0);
-      d_fst = realloc<Actor*>(d_fst,n,2*n);
-      d_cur = d_fst+n;
-      d_lst = d_fst+2*n;
+  Space::notice(Actor& a, ActorProperty p, bool duplicate) {
+    if (p & AP_DISPOSE) {
+      if (duplicate && (d_fst != NULL)) {
+        for (Actor** f = d_fst; f < d_cur; f++)
+          if (&a == *f)
+            return;
+      }
+      if (d_cur == d_lst) {
+        // Resize
+        if (d_fst == NULL) {
+          // Create new array
+          d_fst = alloc<Actor*>(4);
+          d_cur = d_fst;
+          d_lst = d_fst+4;
+        } else {
+          // Resize existing array
+          unsigned int n = static_cast<unsigned int>(d_lst - d_fst);
+          assert(n != 0);
+          d_fst = realloc<Actor*>(d_fst,n,2*n);
+          d_cur = d_fst+n;
+          d_lst = d_fst+2*n;
+        }
+      }
+      *(d_cur++) = &a;
+    } else if (p & AP_WEAKLY) {
+      if (wmp() == 0)
+        wmp(2);
+      else
+        wmp(wmp()+1);
+    }
+  }
+
+  void
+  Space::ignore(Actor& a, ActorProperty p, bool duplicate) {
+    if (p & AP_DISPOSE) {
+      // Check wether array has already been discarded as space
+      // deletion is already in progress
+      if (d_fst == NULL)
+        return;
+      Actor** f = d_fst;
+      if (duplicate) {
+        while (f < d_cur)
+          if (&a == *f)
+            break;
+          else
+            f++;
+        if (f == d_cur)
+          return;
+      } else {
+        while (&a != *f)
+          f++;
+      }
+      *f = *(--d_cur);
+    } else if (p & AP_WEAKLY) {
+      assert(wmp() > 1U);
+      wmp(wmp()-1);
     }
   }
 
@@ -348,7 +409,7 @@ namespace Gecode {
   void
   Space::_commit(const Choice& c, unsigned int a) {
     if (a >= c.alternatives())
-      throw SpaceIllegalAlternative();
+      throw SpaceIllegalAlternative("Space::commit");
     if (failed())
       return;
     if (Brancher* b = brancher(c._id)) {
@@ -361,6 +422,36 @@ namespace Gecode {
     }
   }
 
+  NGL*
+  Space::ngl(const Choice& c, unsigned int a) {
+    if (a >= c.alternatives())
+      throw SpaceIllegalAlternative("Space::ngl");
+    if (failed())
+      return NULL;
+    if (Brancher* b = brancher(c._id)) {
+      // There is a matching brancher
+      return b->ngl(*this,c,a);
+    } else {
+      // There is no matching brancher!
+      throw SpaceNoBrancher("Space::ngl");
+    }
+  }
+
+  void
+  Space::print(const Choice& c, unsigned int a, std::ostream& o) const {
+    if (a >= c.alternatives())
+      throw SpaceIllegalAlternative("Space::print");
+    if (failed())
+      return;
+    if (Brancher* b = const_cast<Space&>(*this).brancher(c._id)) {
+      // There is a matching brancher
+      b->print(*this,c,a,o);
+    } else {
+      // There is no matching brancher!
+      throw SpaceNoBrancher("Space::print");
+    }
+  }
+
   void
   Space::kill_brancher(unsigned int id) {
     if (failed())
@@ -542,7 +633,8 @@ namespace Gecode {
   }
 
   void
-  Space::master(unsigned long int, const Space*) {
+  Space::master(unsigned long int, const Space*, NoGoods& ng) {
+    ng.post(*this);
   }
 
   void
@@ -571,6 +663,18 @@ namespace Gecode {
     gafc.decay(d);
   }
 
+  void
+  Space::afc_set(double a) {
+    afc_enable();
+    for (Propagators p(*this); p(); ++p)
+      gafc.set(p.propagator().gafc,a);
+  }
+
+
+  bool
+  NGL::notice(void) const {
+    return false;
+  }
 
 }
 
